/**
 * Logger for task runner.
 * @memberof module:apeman-commons-logging/lib
 * @inner
 * @function apemanTaskLogger
 * @param {object} [options] - Optional settings.
 * @returns {Logger} - A logger instance.
 */

"use strict";

var Logger = require('./logger'),
    clc = require('cli-color'),
    MessageFormatter = require('./message_formatter'),
    util = require('util');

/** @lends apemanTaskLogger */
function apemanTaskLogger(options) {
    var name = options.name || 'apeman-task',
        verbose = !!options.verbose;
    return new Logger({
        verbose: verbose,
        PREFIX: util.format('[%s] ', name),
        /**
         * Log about task started.
         * @param {string} name - Name of task.
         * @param {object} task - Task data.
         */
        logTaskStarted: function (name, task) {
            var s = this;
            s.info('Task "%s" started...', name);
            if (s.verbose) {
                s.trace(task);
            }
        },
        /**
         * Log about task finished.
         * @param {number} took - Time took.
         */
        logTaskFinished: function (took) {
            var s = this;
            s.info('...done! (%dms)', took);
        },
        /**
         * Log about task finished.
         * @param {Error} err - Task error.
         */
        logTaskFailed: function (err) {
            var s = this;
            s.error(err.stack || err.msg || err);
        },
        /**
         * Log task list.
         * @param {object} tasks - Tasks to show.
         */
        logList: function (tasks) {
            var s = this;
            s.debug('Available tasks:');
            var PREFIX = s.PREFIX;
            s.PREFIX = '';
            Object.keys(tasks).forEach(function (name) {
                var desc = s._descTask(tasks[name]);
                var space = ' ';
                while (name.length + space.length < 20) {
                    space += ' ';
                }
                s.debug("%s%s", clc.black.bold(name), clc.white([space, '//', desc].join(' ')));
            });
            s.debug('');
            s.PREFIX = PREFIX;
        },
        _descTask: function (task) {
            var s = this;
            if (Array.isArray(task)) {
                return task.map(function (name) {
                    return s._descTask(name) || String(name);
                }).join(', ');
            }
            if (typeof(task) === 'function') {
                return util.format('function(){/*...*/}');
            }
            return task.desc || '';
        }
    });
}

module.exports = apemanTaskLogger;
